@@ -1,6 +1,6 @@
 /* Python interface to inferior exit events.
 
-   Copyright (C) 2009-2012 Free Software Foundation, Inc.
+   Copyright (C) 2009-2017 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,38 +17,38 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
+#include "defs.h"
 #include "py-event.h"
 
-static PyTypeObject exited_event_object_type;
+extern PyTypeObject exited_event_object_type
+    CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("event_object");
 
 static PyObject *
 create_exited_event_object (const LONGEST *exit_code, struct inferior *inf)
 {
-  PyObject *exited_event;
-  PyObject *inf_obj;
+  gdbpy_ref<> exited_event (create_event_object (&exited_event_object_type));
 
-  exited_event = create_event_object (&exited_event_object_type);
+  if (exited_event == NULL)
+    return NULL;
 
-  if (!exited_event)
-    goto fail;
+  if (exit_code)
+    {
+      gdbpy_ref<> exit_code_obj (PyLong_FromLongLong (*exit_code));
 
-  if (exit_code
-      && evpy_add_attribute (exited_event,
-			     "exit_code",
-			     PyLong_FromLongLong (*exit_code)) < 0)
-    goto fail;
+      if (exit_code_obj == NULL)
+	return NULL;
+      if (evpy_add_attribute (exited_event.get (), "exit_code",
+			      exit_code_obj.get ()) < 0)
+	return NULL;
+    }
 
-  inf_obj = inferior_to_inferior_object (inf);
-  if (!inf_obj || evpy_add_attribute (exited_event,
-                                      "inferior",
-                                      inf_obj) < 0)
-    goto fail;
+  gdbpy_ref<> inf_obj (inferior_to_inferior_object (inf));
+  if (inf_obj == NULL || evpy_add_attribute (exited_event.get (),
+					     "inferior",
+					     inf_obj.get ()) < 0)
+    return NULL;
 
-  return exited_event;
-
-  fail:
-   Py_XDECREF (exited_event);
-   return NULL;
+  return exited_event.release ();
 }
 
 /* Callback that is used when an exit event occurs.  This function
@@ -57,15 +57,13 @@ create_exited_event_object (const LONGEST *exit_code, struct inferior *inf)
 int
 emit_exited_event (const LONGEST *exit_code, struct inferior *inf)
 {
-  PyObject *event;
-
   if (evregpy_no_listeners_p (gdb_py_events.exited))
     return 0;
 
-  event = create_exited_event_object (exit_code, inf);
+  gdbpy_ref<> event (create_exited_event_object (exit_code, inf));
 
-  if (event)
-    return evpy_emit_event (event, gdb_py_events.exited);
+  if (event != NULL)
+    return evpy_emit_event (event.get (), gdb_py_events.exited);
 
   return -1;
 }
@@ -75,5 +73,4 @@ GDBPY_NEW_EVENT_TYPE (exited,
                       "gdb.ExitedEvent",
                       "ExitedEvent",
                       "GDB exited event object",
-                      event_object_type,
-                      static);
+                      event_object_type);
