@@ -1,7 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 1997, 1998, 2004, 2007, 2008, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright 1997-2017 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,14 +13,10 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
-   */
-
-
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 // Test file for exception handling support.
 
-#include <iostream>
 using namespace std;
 
 int foo (int i)
@@ -36,10 +31,14 @@ extern "C" int bar (int k, unsigned long eharg, int flag);
     
 int bar (int k, unsigned long eharg, int flag)
 {
-  cout << "k is " << k << " eharg is " << eharg << " flag is " << flag << endl;
   return 1;
 }
 
+int catcher (int x)
+{
+  return x;
+}
+
 int main()
 {
   int j;
@@ -48,7 +47,7 @@ int main()
     j = foo (20);
   }
   catch (int x) {
-    cout << "Got an except " << x << endl;
+    catcher (x);
   }
   
   try {
@@ -56,12 +55,12 @@ int main()
       j = foo (20);
     }
     catch (int x) {
-      cout << "Got an except " << x << endl;
+      catcher (x);
       throw;
     }
   }
   catch (int y) {
-    cout << "Got an except (rethrown) " << y << endl;
+    catcher (y);
   }
 
   // Not caught 
