@@ -1,6 +1,6 @@
 /* rl78.c --- opcode semantics for stand-alone RL78 simulator.
 
-   Copyright (C) 2008-2015 Free Software Foundation, Inc.
+   Copyright (C) 2008-2017 Free Software Foundation, Inc.
    Contributed by Red Hat, Inc.
 
    This file is part of the GNU simulators.
@@ -421,10 +421,16 @@ decode_opcode (void)
   int a, b, v, v2;
   unsigned int u, u2;
   int obits;
+  RL78_Dis_Isa isa;
+
+  isa = (rl78_g10_mode ? RL78_ISA_G10
+	: g14_multiply ? RL78_ISA_G14
+	: g13_multiply ? RL78_ISA_G13
+	: RL78_ISA_DEFAULT);
 
   rl78_data.dpc = pc;
   opcode_size = rl78_decode_opcode (pc, &opcode,
-				    rl78_get_byte, &rl78_data);
+				    rl78_get_byte, &rl78_data, isa);
 
   opcode_pc = pc;
   pc += opcode_size;
@@ -851,7 +857,7 @@ decode_opcode (void)
 
       rl78_data.dpc = pc;
       opcode_size = rl78_decode_opcode (pc, &opcode,
-					rl78_get_byte, &rl78_data);
+					rl78_get_byte, &rl78_data, isa);
       pc += opcode_size;
       tprintf (" skipped: %s\n", opcode.syntax);
       break;
